﻿using System;
using EmployeeManagement;
using System.Diagnostics;
using System.IO;
using System.Windows.Automation;
using System.Threading;

namespace EmployeeManagementAPI
{
    public class EmployeeManagementApp : IDisposable
    {
        //開始
        public static EmployeeManagementApp Start()
        {
#if DEBUG
            string path = Path.GetFullPath("../../../bin/Debug/EmployeeManagement.exe");
#else
            string path = Path.GetFullPath("../../../bin/Release/EmployeeManagement.exe");
#endif
            return new EmployeeManagementApp(Process.Start(path));
        }

        AutomationElement mainForm;
        int processId;
        ValuePattern textBoxName;
        ValuePattern textBoxAge;
        ValuePattern textBoxPosition;
        InvokePattern buttonAdd;
        InvokePattern buttonFind;

        //コンストラクタ
        EmployeeManagementApp(Process process)
        {
            while (process.MainWindowHandle == IntPtr.Zero)
            {
                Thread.Sleep(10);
            }
            processId = process.Id;

            //GUIマッピング
            mainForm = AutomationElement.FromHandle(process.MainWindowHandle);
            textBoxName = (ValuePattern)GetPattern("textBoxName", ValuePattern.Pattern);
            textBoxAge = (ValuePattern)GetPattern("textBoxAge", ValuePattern.Pattern);
            textBoxPosition = (ValuePattern)GetPattern("textBoxPosition", ValuePattern.Pattern);
            buttonAdd = (InvokePattern)GetPattern("buttonAdd", InvokePattern.Pattern);
            buttonFind = (InvokePattern)GetPattern("buttonFind", InvokePattern.Pattern);
        }

        //終了
        public void Dispose()
        {
            Process.GetProcessById(processId).CloseMainWindow();
        }

        //登録
        public bool Add(string name, string age, string post)
        {
            textBoxName.SetValue(name);
            textBoxAge.SetValue(age);
            textBoxPosition.SetValue(post);
            buttonAdd.Invoke();
            return string.IsNullOrEmpty(textBoxName.Current.Value);
        }

        //検索
        public EmployeeData Find(string name)
        {
            //検索
            textBoxName.SetValue(name);
            buttonFind.Invoke();

            //データ作成
            if (string.IsNullOrEmpty(textBoxName.Current.Value))
            {
                return null;
            }
            return new EmployeeData(textBoxName.Current.Value,
                int.Parse(textBoxAge.Current.Value), 
                textBoxPosition.Current.Value);            
        }

        //パターンの取得
        private object GetPattern(string name, AutomationPattern pattern)
        {
            AutomationElement textElement = mainForm.FindFirst(
                TreeScope.Element | TreeScope.Descendants,
                new PropertyCondition(
                  AutomationElement.AutomationIdProperty,
                  name));
            return textElement.GetCurrentPattern(pattern);
        }
    }
}
